﻿//////////////////////////////////////////////
// ContentLoader.h
//
//////////////////////////////////////////////

/// Defines / macro --------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class Compositor ;
}

namespace nkExport
{
	class Node ;
	class Exportable ;
}

/// Includes ---------------------------------

// nkGraphics
#include "ContentLoadResult.h"

#include "../Dll/DllDefines.h"

// nkExport
#include <NilkinsExport/Exporter/ExporterLoadResult.h>

// Standards
#include <string_view>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ContentLoader final
	{
		private :

			// Constructeur, destructeur
			ContentLoader () = delete ;
			~ContentLoader () = delete ;
			
		public :

			// Fonctions pratiques
			static nkExport::ExporterLoadResult inputData (const std::string_view& filePath) ;
			static std::string inputDataStr (const std::string_view& filePath) ;
			static nkExport::ExporterLoadResult inputDataSources (const std::string_view& sources) ;
			static bool outputData (const std::string_view& content, const std::string_view& path) ;
		
			// Save
			static bool saveTreeDeclaration (nkExport::Node* tree, const std::string_view& pathFromData) ;
			static bool saveObjectDeclaration (nkExport::Exportable* object, const std::string_view& pathFromData) ;

			// Les fonctions spéciales pour chaque type
			// Elles concernent le chargement du coup
			// Pour les textures
			static ContentLoadResultTexture loadTextureDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultTexture processTextureSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultTexture processTextureTree (nkExport::Node* tree, bool loadResource = true) ;
			// Samplers
			static ContentLoadResultSampler loadSamplerDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultSampler processSamplerSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultSampler processSamplerTree (nkExport::Node* tree, bool loadResource = true) ;
			// Shaders Program
			static ContentLoadResultProgram loadShaderProgramDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultProgram processShaderProgramSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultProgram processShaderProgramTree (nkExport::Node* tree, bool loadResource = true) ;
			// Shader Memory Slot
			static ContentLoadResultShaderMemorySlot loadShaderMemorySlotDeclaration (const std::string_view& pathFromData) ;
			static ContentLoadResultShaderMemorySlot processShaderMemorySlotSources (const std::string_view& sources) ;
			static ContentLoadResultShaderMemorySlot processShaderMemorySlotDeclarationTree (nkExport::Node* tree) ;
			// Constant Buffer
			static ContentLoadResultConstantBuffer loadConstantBufferDeclaration (const std::string_view& pathFromData) ;
			static ContentLoadResultConstantBuffer processConstantBufferSources (const std::string_view& sources) ;
			static ContentLoadResultConstantBuffer processConstantBufferDeclarationTree (nkExport::Node* tree) ;
			// Compute Buffer
			static ContentLoadResultBuffer loadBufferDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultBuffer processBufferSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultBuffer processBufferDeclarationTree (nkExport::Node* tree, bool loadResource = true) ;
			// Shader
			static ContentLoadResultShader loadShaderDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultShader processShaderDeclarationSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultShader processShaderDeclarationTree (nkExport::Node* tree, bool loadResource = true) ;
			// Compositor
			static ContentLoadResultCompositor loadCompositorDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultCompositor processCompositorSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultCompositor processCompositorTree (nkExport::Node* tree, bool loadResource = true) ;
			// Meshes
			static ContentLoadResultMesh loadMeshDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultMesh processMeshSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultMesh processMeshTree (nkExport::Node* tree, bool loadResource = true) ;
			// Nodes
			static ContentLoadResultNode loadNodeDeclaration (const std::string_view& pathFromData) ;
			static ContentLoadResultNode processNodeSources (const std::string_view& sources) ;
			static ContentLoadResultNode processNodeTree (nkExport::Node* tree) ;
			// Blend States
			static ContentLoadResultBlendState loadBlendStateDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultBlendState processBlendStateSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultBlendState processBlendStateTree (nkExport::Node* tree, bool loadResource = true) ;
			// Depth Stencil States
			static ContentLoadResultDepthStencilState loadDepthStencilStateDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultDepthStencilState processDepthStencilStateSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultDepthStencilState processDepthStencilStateTree (nkExport::Node* tree, bool loadResource = true) ;
			// Raster States
			static ContentLoadResultRasterState loadRasterStateDeclaration (const std::string_view& pathFromData, bool loadResource = true) ;
			static ContentLoadResultRasterState processRasterStateSources (const std::string_view& sources, bool loadResource = true) ;
			static ContentLoadResultRasterState processRasterStateTree (nkExport::Node* tree, bool loadResource = true) ;
			// Camera
			static ContentLoadResultCamera loadCameraDeclaration (const std::string_view& pathFromData) ;
			static ContentLoadResultCamera processCameraSources (const std::string_view& sources) ;
			static ContentLoadResultCamera processCameraTree (nkExport::Node* tree) ;
			// Render Queues
			static ContentLoadResultRenderQueue loadRenderQueueDeclaration (const std::string_view& pathFromData) ;
			static ContentLoadResultRenderQueue processRenderQueueSources (const std::string_view& sources) ;
			static ContentLoadResultRenderQueue processRenderQueueTree (nkExport::Node* tree) ;

			// Pour des objets dépendants d'autres
			// Compositor Node
			static ContentLoadResultCompositorNode appendCompositorNodeFromFile (const std::string_view& pathFromData, Compositor* parent) ;
			static ContentLoadResultCompositorNode appendCompositorNodeFromSources (const std::string_view& sources, Compositor* parent) ;
			static ContentLoadResultCompositorNode appendCompositorNodeFromTree (nkExport::Node* tree, Compositor* parent) ;
	} ;
}