﻿//////////////////////////////////////////////
// ShaderManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Shader ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Utilities/InfiniteMask.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ShaderManager final : public nkCommon::SingletonClass<ShaderManager>
	{
		friend class nkCommon::SingletonClass<ShaderManager> ;

		private :
		
			// Attributs
			// Mutex unique pour tout, à voir si bénéfice à split
			std::mutex _mutex ;

			// Mémoire des shaders instanciés
			std::unordered_map<std::string, Shader*> _shaderMap ;

			// Compteurs pour les ids
			unsigned int _shaderCounter ;

		private :

			// Functions
			// Constructeur
			ShaderManager () ;
		
		public :

			// Destructeur
			~ShaderManager () ;

			// Getters
			bool isBuiltInShader (Shader* toTest) ;

			// Manipulations de shader
			Shader* createOrRetrieve (const std::string_view& name) ;
			Shader* get (const std::string_view& name) ;
			Shader* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;
	} ;
}