﻿//////////////////////////////////////////////
// Script.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class ScriptStrategy ;

	enum class INTERPRETER ;

	struct CompilationResult ;
}

/// Includes ---------------------------------

// nkScripts
#include "../Dll/DllDefines.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <string>

/// Class ------------------------------------

namespace nkScripts
{
	class DLL_SCRIPTS_EXPORT Script final : public nkExport::Exportable
	{
		friend class EnvironmentStrategy ;

		private :

			// Attributes
			// Sources actives
			std::string _sources ;
			// Nom
			std::string _name ;

			// Strategy attachée
			ScriptStrategy* _script ;
			// Type de stript
			INTERPRETER _scriptType ;

		private :

			// Functions
			// Getters
			ScriptStrategy* _getStrategy () const ;

		public :

			// Constructeur, destructeur
			Script () ;
			~Script () ;

			// Getters
			INTERPRETER getScriptType () const ;
			std::string_view getName () const ;
			std::string_view getSources () const ;
			const CompilationResult& getLastCompilationResult () const ;

			// Setters
			void setScriptFor (INTERPRETER interpretType) ;
			void setName (const std::string_view& name) ;
			void setSources (const std::string_view& sources) ;

			// Loading
			bool load () ;
			void unload () ;

			// Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}
