﻿//////////////////////////////////////////////
// WindowingSystem.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class Component ;
	class ComponentUtils ;
	class DataAllocator ;
	class InputProcessor ;

	enum class POPUP_RESULT ;
	enum class POPUP_TYPE ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT WindowingSystem
	{
		protected :

			// Attributes
			// Allocator pour les components
			DataAllocator* _allocator ;

			// Utils pour les manipuler
			ComponentUtils* _componentUtils ;

			// Input system propre
			InputProcessor* _inputProcessor ;

		public :

			// Constructeur, destructeur
			WindowingSystem () ;
			virtual ~WindowingSystem () ;

			// Initialisation
			virtual bool load () = 0 ;

			// Run
			virtual void run () = 0 ;
			virtual void tick () = 0 ;

			// Shut down
			virtual void prepareForShutdown () = 0 ;
			virtual void shutdown () = 0 ;

			// Poke
			virtual void notifyEvent () = 0 ;

			// Popup éphémères
			virtual POPUP_RESULT staticPopup (POPUP_TYPE type, const char* title, const char* message, Component* parent) = 0 ;

			// Getters
			DataAllocator* getDataAllocator () const ;
			ComponentUtils* getComponentUtils () const ;
			InputProcessor* getInputProcessor () const ;
	} ;
}